#pragma once

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/PriorKnowledge/core/FinderBase.h>
#include <RobotAPI/libraries/PriorKnowledge/core/FinderInfoBase.h>

namespace armarx::priorknowledge::core
{
    /**
   * @brief The PKDatasetFinderInfo class
   * Specialization of the DatasetFinderInfo with strings as dataset and id types
   */
    class PKDatasetFinderInfo : public core::DatasetFinderInfoBase<std::string, std::string>
    {
        using Base = core::DatasetFinderInfoBase<std::string, std::string>;

    public:
        using Base::Base;
    };

    /**
     * @brief The PKDatasetFinder class
     */
    class PKDatasetFinder :
        public core::DatasetFinderBase<std::string, std::string, PKDatasetFinderInfo>
    {
        using Base = core::DatasetFinderBase<std::string, std::string, PKDatasetFinderInfo>;

    public:
        static const constexpr auto DEFAULT_PACKAGE = "PriorKnowledgeData";

        using Base::Base;

        PKDatasetFinder(const std::filesystem::path& relDir) : Base(DEFAULT_PACKAGE, relDir)
        {
        }

        virtual ~PKDatasetFinder()
        {
        }

        virtual void init();
        virtual void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& defs,
                                                 const std::string& prefix);

        bool checkAll(const std::string& dataset) const override;
        bool checkAll() const override;

        bool check(const std::string& id) const override;
        bool check(const std::string& dataset, const std::string& id) const override;

        std::optional<PKDatasetFinderInfo> find(const std::string& dataset,
                                                const std::string& id) const override;
        std::optional<PKDatasetFinderInfo> find(const std::string& id) const override;

        std::vector<PKDatasetFinderInfo> findAll() const override;

        std::vector<PKDatasetFinderInfo> findAll(const std::string& dataset) const override;

    private:
        std::vector<std::string> DATASET_FOLDERS_BLACKLIST = {};
        std::vector<std::string> ID_BLACKLIST = {"script"};
    };

} // namespace armarx::priorknowledge::core
