#pragma once

#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/PriorKnowledge/core/FinderBase.h>
#include <RobotAPI/libraries/PriorKnowledge/core/FinderInfoBase.h>

namespace armarx::priorknowledge::core
{
    /**
   * @brief The ObjectFinderInfo class
   * Specialization of the DatasetFinderInfo with strings as dataset and id types
   */
    class PKFinderInfo : public core::FinderInfoBase<std::string>
    {
        using Base = core::FinderInfoBase<std::string>;

    public:
        using Base::Base;
    };

    /**
     * @brief The ObjectFinder class
     */
    class PKFinder : public core::FinderBase<std::string, PKFinderInfo>
    {
        using Base = core::FinderBase<std::string, PKFinderInfo>;

    public:
        static const constexpr auto DEFAULT_PACKAGE = "PriorKnowledgeData";

        using Base::Base;

        PKFinder(const std::filesystem::path& relDir) : Base(DEFAULT_PACKAGE, relDir)
        {
        }

        virtual ~PKFinder()
        {
        }

        virtual void init();
        virtual void registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& defs,
                                                 const std::string& prefix);

        bool checkAll() const override;

        bool check(const std::string& id) const override;

        std::optional<PKFinderInfo> find(const std::string& id) const override;

        std::vector<PKFinderInfo> findAll() const override;

    private:
        std::vector<std::string> ID_BLACKLIST = {"script"};
    };

} // namespace armarx::priorknowledge::core
