#pragma once

#include <map>
#include <mutex>
#include <optional>
#include <string>

#include <RobotAPI/libraries/armem/server/ltm/processors/Processors.h>

namespace armarx::armem::server::ltm::detail
{
    /// @brief Interface functions for the longterm memory classes
    class MemoryItem
    {
    public:
        MemoryItem(const std::string& exportName, const MemoryID&); // only used by memory
        MemoryItem(const std::string& exportName,
                   const MemoryID&,
                   const std::shared_ptr<Processors>&); // used by all other segments
        virtual ~MemoryItem() = default;

        void setExportName(const std::string& n);
        void setMemoryID(const MemoryID&);
        void setMemoryName(const std::string& memoryName);

        std::string
        getExportName() const
        {
            return exportName;
        }

        MemoryID
        getMemoryID() const
        {
            return id();
        }

        // aliases
        MemoryID id() const;
        std::string name() const;

    protected:
        virtual void
        _setExportName(const std::string&)
        {
        }

        virtual void
        _setMemoryID(const MemoryID&)
        {
        }

    protected:
        std::shared_ptr<Processors> processors;

    private:
        std::string exportName = "MemoryExport";
        MemoryID _id;
    };
} // namespace armarx::armem::server::ltm::detail
