/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::aron
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2019
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::aron

#define ARMARX_BOOST_TEST

// STD/STL
#include <iostream>
#include <cstdlib>
#include <ctime>
#include <numeric>

// Test
#include <RobotAPI/Test.h>

// Aron
#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/codegeneration/test/aron/HumanPoseTest.aron.generated.h>


using namespace armarx;
using namespace aron;

namespace armarx
{
    class HumanPoseBO
    {
    public:
        std::vector<float> jointValues;
        bool reached;

        HumanPoseBO() = default;

        void fromAron(const HumanPoseTest& aron)
        {
            jointValues = aron.jointValues;
            reached = aron.reached;
        }

        HumanPoseTest toAron()
        {
            HumanPoseTest aron;
            aron.jointValues = jointValues;
            aron.reached = reached;
            return aron;
        }
    };

    void fromAron(const HumanPoseTest& aron, HumanPoseBO& bo)
    {
        bo.jointValues = aron.jointValues;
        bo.reached = aron.reached;
    }

    void toAron(HumanPoseTest& aron, const HumanPoseBO& bo)
    {
        aron.jointValues = bo.jointValues;
        aron.reached = bo.reached;
    }

    BOOST_AUTO_TEST_CASE(AronAssignmentTest)
    {
        armarx::HumanPoseTest aron;
        aron.jointValues = {1,2,3,4};
        aron.reached = true;

        HumanPoseBO bo;
        aron.to(bo);

        BOOST_CHECK((aron.jointValues == bo.jointValues));

        HumanPoseBO bo2;
        aron.to(bo, &fromAron);

        BOOST_CHECK((aron.jointValues == bo2.jointValues));
    }
}
